#    Munakello - Simple Pomodoro timer
#    Copyright (C) 2016 Timo Kankare 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import asyncio
from datetime import datetime, timedelta

def beep():
    print('\a', end='', flush=True)

def round_time(time, precision):
    return precision * ((time + precision / 2) // precision)

def calculate_next_beep_time(remaining_time, end_time):
    time_scales = [
        timedelta(seconds=60),
        timedelta(seconds=10),
        timedelta(seconds=3),
        timedelta(seconds=2),
        timedelta(seconds=1),
    ]
    for nextstep in time_scales:
        if remaining_time > nextstep:
            return end_time - nextstep
    else:
        return end_time

async def report_beep(start_time, end_time):
    try:
        while True:
            now = datetime.now()
            remaining_time = end_time - now
            if remaining_time.total_seconds() <= 0:
                break
            next_time = calculate_next_beep_time(remaining_time, end_time)
            await asyncio.sleep((next_time - now).total_seconds())
            beep()
    except asyncio.CancelledError:
        print('Beep reporting canceled')

def select_message(remaining_time):
    time_scales = [
        (timedelta(seconds=59), '{minutes} minutes remaining'),
        (timedelta(seconds=0), '{seconds} seconds remaining'),
    ]
    for (limit, message) in time_scales:
        if remaining_time > limit:
            return message
    return '{time} remaining'

def format_time(remaining_time):
    message = select_message(remaining_time)
    hours = round(remaining_time.total_seconds() / (60*60))
    minutes = round(remaining_time.total_seconds() / 60)
    seconds = round(remaining_time.total_seconds())
    return message.format(hours=hours, minutes=minutes, seconds=seconds, time=remaining_time)

def calculate_next_report_time(remaining_time, end_time):
    time_scales = [
        (timedelta(seconds=60), timedelta(seconds=60)),
        (timedelta(seconds=10), timedelta(seconds=10)),
        (timedelta(seconds=0),  timedelta(seconds=1)),
    ]
    for (limit, nextstep) in time_scales:
        t = round_time(remaining_time - nextstep, nextstep)
        if t >= limit:
            return end_time - t
    return end_time

async def report_time(start_time, end_time):
    try:
        while True:
            now = datetime.now()
            remaining_time = end_time - now
            if remaining_time.total_seconds() <= 0:
                break
            print(format_time(remaining_time))
            next_time = calculate_next_report_time(remaining_time, end_time)
            await asyncio.sleep((next_time - now).total_seconds())
    except asyncio.CancelledError:
        print('Time reporting canceled')

async def wait_delay(delay, reporters):
    start_time = datetime.now()
    end_time = start_time + timedelta(seconds=delay)

    # start reporting tasks
    reporter_tasks = [asyncio.ensure_future(reporter(start_time, end_time)) for reporter in reporters]

    # wait end of delay
    await asyncio.sleep(delay)

    # wait tasks to end
    (done, pending) = await asyncio.wait(reporter_tasks, timeout=1)
    # cancel remaining tasks
    for task in pending:
        task.cancel()

async def work(delay, start_message, stop_message, reporters):
    print(start_message)
    await wait_delay(delay, reporters)
    print(stop_message)

async def run_pomodoro(reporters):
    await work(25*60, 'Start working...', 'Stop working!', reporters)
    await work(5*60, 'Start break...', 'Stop break.', reporters)
    #await work(1*60, 'Start test...', 'Stop test.', reporters)

def main():
    reporters = [report_time, report_beep]
    loop = asyncio.get_event_loop()
    tasks = [ asyncio.ensure_future(run_pomodoro(reporters)) ]
    try:
        loop.run_until_complete(asyncio.wait(tasks))
    finally:
        loop.close()

if __name__ == '__main__':
    main()

